#' Misclassification Corrected Logistic Regression for Bilateral Disease Data
#'
#' Estimate logistic regression for binary worse-entity response defined as
#' y=max(z_1, z_2) based on error-prone single entity disease classifications z1_star and
#' z2_star and gold standard classifications z1 and z2 for a validation subset of individuals.
#' 4 different conditional probabilities are modeled: (i) the conditional probability of
#' worse-entity disease, and (ii) the probability of disease in both entities given disease.
#' Furthermore, (iii) the sensitivity and (iv) the specificity of the entity-specific
#' classification process, i.e. P(Z*=1|Z=1), and P(Z*=0|Z=0).
#' We model all conditional probabilities based on the logistic function, corresponding to a standard
#' logistic regression for worse-entity.
#' Estimation is based on optimization of a likelihood where each of the n observation contributes
#' depending on the available outcome data, i.e. a subset of (z1, z2, z1.star, z2.star).
#'
#' @param z1_star vector of error-prone binary response observations of entity 1 of length n, NA if missing
#' @param z2_star vector of error-prone binary response observations of entity 2 of length n, NA if missing
#' @param z1 vector of binary gold-standard response observations of entity 1 of length n, NA if missing
#' @param z2 vector of binary gold-standard response observations of entity 2 of length n, NA if missing
#' @param formula a formula specifying the design matrix of the logistic regression model
#' with worse-entity response max(z1,z2) as in \code{\link[stats]{model.matrix}}
#' @param formula_delta a formula specifying the design matrix for modeling delta (probability of disease
#' in both entities, given disease)
#' @param formula_sens a formula specifying the person-specific design matrix for modeling the
#' sensitivity of the error-prone entity-specific observation process
#' @param formula_spec a formula specifying the person-specific design matrix for modeling the
#' specificity of the error-prone entity-specific observation process
#' @param data a data.frame with n rows including all terms used in the model formulas
#' @param control list of control parameters for the numerical likelihood optimization as
#' in \code{\link[stats]{optim}}
#' @importFrom stats glm.fit model.matrix optim as.formula binomial pchisq qlogis
#' @import checkmate
#' @export
#'

est_mcblog = function(z1_star,
                    z2_star,
                    z1, z2,
                    formula,
                    formula_delta,
                    formula_sens,
                    formula_spec,
                    data,
                    control = list(maxit = 5000, trace = 0)) {

  n = nrow(data)

  if(is.logical(z1_star))
    z1_star = as.numeric(z1_star)
  if(is.logical(z2_star))
    z2_star = as.numeric(z2_star)
  if(is.logical(z1))
    z1 = as.numeric(z1)
  if(is.logical(z2))
    z2 = as.numeric(z2)

  assert_integerish(z1_star, lower = 0, upper = 1, len = n)
  assert_integerish(z2_star, lower = 0, upper = 1, len = n)
  assert_integerish(z1, lower = 0, upper = 1, len = n)
  assert_integerish(z2, lower = 0, upper = 1, len = n)

  num.outcome.obs = 4 - is.na(z1_star) - is.na(z2_star) - is.na(z1) - is.na(z2)
  assert_integerish(num.outcome.obs, lower = 1)



  des_mat = model.matrix(as.formula(formula), data = data)
  des_mat_delta = model.matrix(as.formula(formula_delta), data = data)
  des_mat_sens = model.matrix(as.formula(formula_sens), data = data)
  des_mat_spec = model.matrix(as.formula(formula_spec), data = data)

  cross_tab = prop.table(table(c(z1, z2), c(z1_star, z2_star)), 1)
  if (length(dim(cross_tab)) == 2 & identical(dim(cross_tab),c(2,2))) {
    sens_start = qlogis(cross_tab[2, 2])
    spec_start = qlogis(cross_tab[1, 1])
  } else {
    sens_start = qlogis(0.9)
    spec_start = qlogis(0.9)
  }
  para_start = c(
    rep(0, times = ncol(des_mat) + ncol(des_mat_delta)),
    sens_start, rep(0, ncol(des_mat_sens) - 1),
    spec_start, rep(0, ncol(des_mat_spec) - 1)
  )
  res = optim(
    par = para_start,
    fn = neg_loglik,
    z1 = z1,
    z2 = z2,
    z1_star = z1_star,
    z2_star = z2_star,
    des_mat = des_mat,
    des_mat_delta = des_mat_delta,
    des_mat_sens = des_mat_sens,
    des_mat_spec = des_mat_spec,
    method = "L-BFGS-B",
    hessian = TRUE,
    control = control
  )
  coef = data.frame(beta = res$par, se = sqrt(diag(solve(res$hessian))))
  rownames(coef) = c(
    paste0("y_", colnames(des_mat)),
    paste0("delta_", colnames(des_mat_delta)),
    paste0("sens_", colnames(des_mat_sens)),
    paste0("spec_", colnames(des_mat_spec))
  )
  coef$t_val = coef$beta / coef$se
  coef$p = pchisq(coef$t_val^2, lower.tail = FALSE, df = 1)

  out = list(coef, res)
  class(out) = "blogmc"
  out
}
