## ---- include = FALSE---------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(knitr)

## ----sampledat, message = FALSE-----------------------------------------------
library(tidyverse)
set.seed(21479)
# Sample standard normal continuous covariate x
x = rnorm(2000, 0, 1)
# Sample true worse-entity outcome y based on logistic regression model with logOR(X)=1
y = rbinom(2000, 1, plogis(x))
# Sample true single-entity outcomes z_1, z_2 based on P(Z_1=1, Z_2=2|Y=1)=delta
delta=.75
# Select all subjects with (sampled) true worse-entity outcome y=1
ind_y_1 = which(y==1)
# Sample fraction of subjects with disease in both entities (z1=1, z2=1) from 
# all subjects with y=1
ind_z1z2_1 = sample(ind_y_1, replace = FALSE, size = floor(delta*length(ind_y_1)))
# Get all individuals with disease in one of two entities
ind_onez_1 = setdiff(ind_y_1, ind_z1z2_1)
# Sample whether z1 or z2 equals 1
ind_z1_1_z2_0 = sample(ind_onez_1, replace = FALSE, size = floor(length(ind_onez_1)/2))
ind_z1_0_z2_1 = setdiff(ind_onez_1, ind_z1_1_z2_0)
# Define true entity-specific observations z1 and z2
z1 = rep(0, 2000)
z2 = rep(0, 2000)
z1[ind_z1z2_1] = 1
z2[ind_z1z2_1] = 1
z1[ind_z1_1_z2_0] = 1
z2[ind_z1_0_z2_1] = 1

## ----sample_err---------------------------------------------------------------
# Sample error-prone single entity ourcomes z*_1, z*_2 based on two different assumptions 
# regarding sensitivity/specificity:
# Scenario 1: sens=spec=.8
z1_star_1 = rep(NA, 2000)
z2_star_1 = rep(NA, 2000)
z1_star_1[which(z1==1)] = rbinom(sum(z1==1), 1, .8)
z1_star_1[which(z1==0)] = rbinom(sum(z1==0), 1, 1-0.8)
z2_star_1[which(z2==1)] = rbinom(sum(z2==1), 1, .8)
z2_star_1[which(z2==0)] = rbinom(sum(z2==0), 1, 1-0.8)
# Scenario2: sens=.8, spec=plogis(1.5+0.5*x)
z1_star_2 = rep(NA, 2000)
z2_star_2 = rep(NA, 2000)
z1_star_2[which(z1==1)] = rbinom(sum(z1==1), 1, .8)
z1_star_2[which(z1==0)] = rbinom(sum(z1==0), 1, 1-plogis(1.5+.5*x[which(z1==0)]))
z2_star_2[which(z2==1)] = rbinom(sum(z2==1), 1, .8)
z2_star_2[which(z2==0)] = rbinom(sum(z2==0), 1, 1-plogis(1.5+.5*x[which(z2==0)]))

## ----sample_err_2-------------------------------------------------------------
# Select 25% of subjects into validation data (i.e., remove true single-entity 
# observations for other 75%)
main_stud_ind = sample(1:2000, 1500, replace = FALSE)
z1[main_stud_ind] = NA
z2[main_stud_ind] = NA
# Remove error-prone single-entity disease of 50 subjects in entity 1 and entity 2
rm_z1star = sample(1:1000, 50, replace = FALSE)
rm_z2star = sample(1001:2000, 50, replace = FALSE)
z1_star_1[rm_z1star] = NA
z1_star_2[rm_z1star] = NA
z2_star_1[rm_z2star] = NA
z2_star_2[rm_z2star] = NA
# Derive error-prone worse-entity ourcome for both scenarios
y_star_1 = pmax(z1_star_1, z2_star_1, na.rm = TRUE)
y_star_2 = pmax(z1_star_2, z2_star_2, na.rm = TRUE)

## ----sample_err_ct------------------------------------------------------------
# Compare true worse-entity disease y and error-prone versions y_star_1/y_star_2
kable(table(y, y_star_1))
kable(table(y, y_star_2))

## ----sample_err_ct_sens-------------------------------------------------------
# Compare true worse-entity disease y and error-prone versions y_star_1/y_star_2
kable(prop.table(table(y, y_star_1), margin = 1), digits = 2)
kable(prop.table(table(y, y_star_2), margin = 1), digits = 2)

## ---- fig.width=8, fig.height=3.5---------------------------------------------
tibble(y, y_star_1, y_star_2, x) %>%
  filter(y==0) %>% group_by(x_cut = cut(x, breaks = seq(-4,4, by = 1))) %>%
  summarise(fp_y_star_1 = mean(y_star_1==1),
            fp_y_star_2 = mean(y_star_2==1)) %>% 
  pivot_longer(cols = c("fp_y_star_1", "fp_y_star_2")) %>%
  ggplot() + geom_bar(aes(x_cut, value, fill = name, group = name), stat="identity", 
                      width=.75, position = "dodge", alpha = .5) +
  xlab("X") + ylab("P(false-positive), person-specific") + 
  theme_bw() 

## ----simplelog, message=FALSE-------------------------------------------------
logreg_true = glm(y~x, family = "binomial")
logreg_y_star_1 = glm(y_star_1~x, family = "binomial")
logreg_y_star_2 = glm(y_star_2~x, family = "binomial")

## ---- message = FALSE---------------------------------------------------------
kable(summary(logreg_true)$coef, digits = 2)
confint(logreg_true)[2,]

## ---- message = FALSE---------------------------------------------------------
kable(summary(logreg_y_star_1)$coef, digits = 2)
confint(logreg_y_star_1)[2,]
kable(summary(logreg_y_star_2)$coef, digits = 2)
confint(logreg_y_star_2)[2,]

## -----------------------------------------------------------------------------
library(mcblog)
# Estimate MC-adjusted logistic regression assuming constant
# delta, sens, and spec estimated from validation data
# Scenario 1
mla1_y_star_1 = est_mcblog(z1_star = z1_star_1, 
                           z2_star = z2_star_1, 
                           z1 = z1, 
                           z2 = z2, 
                           formula = ~x,
                           formula_delta = ~1, #constant delta, sens, spec
                           formula_sens = ~1,
                           formula_spec = ~1, 
                           data = data.frame(x))
# There exists also an mcblog:::summary.blogmc method, but for compilation it is better
# to directly access the estimated coefficients in the blogmc objects (first list entry)
kable(mla1_y_star_1[[1]], digits = 2)

## -----------------------------------------------------------------------------
# Scenario 2
mla1_y_star_2 = est_mcblog(z1_star = z1_star_2, 
                           z2_star = z2_star_2, 
                           z1 = z1, 
                           z2 = z2, 
                           formula = ~x,
                           formula_delta = ~1,
                           formula_sens = ~1,
                           formula_spec = ~1, 
                           data = data.frame(x))
kable(mla1_y_star_2[[1]], digits = 2)

## -----------------------------------------------------------------------------
# Estimate MC-adjusted logistic regression assuming varying
# sens, and spec estimated from validation data and constant delta
# Scenario 1
mla2_y_star_1 = est_mcblog(z1_star = z1_star_1, 
                            z2_star = z2_star_1, 
                            z1 = z1, 
                            z2 = z2, 
                            formula = ~x,
                            formula_delta = ~1,
                            formula_sens = ~x,
                            formula_spec = ~x, 
                            data = data.frame(x))
kable(mla2_y_star_1[[1]], digits = 2)

## -----------------------------------------------------------------------------
# Scenario 2
mla2_y_star_2 = est_mcblog(z1_star = z1_star_2, 
                            z2_star = z2_star_2, 
                            z1 = z1, 
                            z2 = z2, 
                            formula = ~x,
                            formula_delta = ~1,
                            formula_sens = ~x,
                            formula_spec = ~x, 
                            data = data.frame(x))
kable(mla2_y_star_2[[1]], digits = 2)

